@@ -332,8 +332,13 @@ class NilmDB():
             query += " AND streams.layout=?"
             params += (layout,)
         if path is not None:
-            query += " AND streams.path=?"
-            params += (path,)
+            if type(path) is list:
+                subquery = " OR ".join(["streams.path LIKE ?" for _ in range(len(path))])
+                query += f" AND ({subquery})"
+                params += tuple(path)
+            else:
+                query += " AND streams.path LIKE ?"
+                params += (path,)
         query += " GROUP BY streams.id ORDER BY streams.path"
         result = self.con.execute(query, params).fetchall()
         return [list(x) for x in result]
