@@ -4,6 +4,7 @@ import os
 import json
 import socket
 import traceback
+import atexit
 
 import psutil
 import cherrypy
@@ -23,7 +24,7 @@ from nilmdb.server.serverutil import (
     cherrypy_start,
     cherrypy_stop,
     bool_param,
-    )
+)
 
 # Add CORS_allow tool
 cherrypy.tools.CORS_allow = cherrypy.Tool('on_start_resource', CORS_allow)
@@ -37,6 +38,7 @@ class NilmApp():
 # CherryPy apps
 class Root(NilmApp):
     """Root application for NILM database"""
+
     # /
     @cherrypy.expose
     def index(self):
@@ -109,7 +111,8 @@ class Stream(NilmApp):
     def list(self, path=None, layout=None, extended=None):
         """List all streams in the database.  With optional path or
         layout parameter, just list streams that match the given path
-        or layout.
+        or layout. May specify multiple paths by adding path more than once
+        to the query. Path parameter accepts SQL wildcards (%,_)
 
         If extended is missing or zero, returns a list of lists
         containing the path and layout: [ path, layout ]
@@ -238,8 +241,8 @@ class Stream(NilmApp):
         if binary and content_type:
             if content_type != "application/octet-stream":
                 raise cherrypy.HTTPError("400", "Content type must be "
-                                         "application/octet-stream for "
-                                         "binary data, not " + content_type)
+                                                "application/octet-stream for "
+                                                "binary data, not " + content_type)
 
         # Note that non-binary data is *not* decoded from bytes to string,
         # but rather passed directly to stream_insert.
@@ -289,6 +292,7 @@ class Stream(NilmApp):
                 if restart is None:
                     break
                 start = restart
+
         return content(start, end)
 
     # /stream/intervals?path=/newton/prep
@@ -330,6 +334,7 @@ class Stream(NilmApp):
                 if restart is None:
                     break
                 start = restart
+
         return content(start, end)
 
     # /stream/extract?path=/newton/prep&start=1234567890.0&end=1234567899.0
@@ -367,7 +372,7 @@ class Stream(NilmApp):
             content_type = "application/octet-stream"
             if markup or count:
                 raise cherrypy.HTTPError("400", "can't mix binary and "
-                                         "markup or count modes")
+                                                "markup or count modes")
         else:
             content_type = "text/plain"
         cherrypy.response.headers['Content-Type'] = content_type
@@ -389,11 +394,13 @@ class Stream(NilmApp):
                 if restart is None:
                     return
                 start = restart
+
         return content(start, end)
 
 
 class Exiter():
     """App that exits the server, for testing"""
+
     @cherrypy.expose
     def index(self):
         cherrypy.response.headers['Content-Type'] = 'text/plain'
@@ -403,15 +410,16 @@ class Exiter():
             raise SystemExit
 
         return content()
+
     index._cp_config = {'response.stream': True}
 
 
 class Server():
     def __init__(self, db, host='127.0.0.1', port=8080,
-                 stoppable=False,        # whether /exit URL exists
-                 fast_shutdown=False,    # don't wait for clients to disconn.
+                 stoppable=False,  # whether /exit URL exists
+                 fast_shutdown=False,  # don't wait for clients to disconn.
                  force_traceback=False,  # include traceback in all errors
-                 basepath='',            # base URL path for cherrypy.tree
+                 basepath='',  # base URL path for cherrypy.tree
                  ):
         # Save server version, just for verification during tests
         self.version = nilmdb.__version__
@@ -419,9 +427,9 @@ class Server():
         self.db = db
         if not getattr(db, "_thread_safe", None):
             raise KeyError("Database object " + str(db) + " doesn't claim "
-                           "to be thread safe.  You should pass "
-                           "nilmdb.utils.serializer_proxy(NilmDB)(args) "
-                           "rather than NilmDB(args).")
+                                                          "to be thread safe.  You should pass "
+                                                          "nilmdb.utils.serializer_proxy(NilmDB)(args) "
+                                                          "rather than NilmDB(args).")
 
         # Build up global server configuration
         cherrypy.config.update({
@@ -429,14 +437,14 @@ class Server():
             'server.socket_host': host,
             'server.socket_port': port,
             'engine.autoreload.on': False,
-            'server.max_request_body_size': 8*1024*1024,
-            })
+            'server.max_request_body_size': 8 * 1024 * 1024,
+        })
 
         # Build up application specific configuration
         app_config = {}
         app_config.update({
             'error_page.default': self.json_error_page,
-            })
+        })
 
         # Some default headers to just help identify that things are working
         app_config.update({'response.headers.X-Jim-Is-Awesome': 'yeah'})
@@ -497,9 +505,21 @@ class Server():
 # instance since the database can only be opened once.  For this to
 # work, the web server must use only a single process and single
 # Python interpreter.  Multiple threads are OK.
+
+## NOTE: For mod_wsgi set threads=1, otherwise multiple process will be spawned
+
 _wsgi_server = None
 
 
+def cleanup():
+    """Ensure the NilmDB instance is properly shutdown and all files are closed"""
+    if _wsgi_server is not None:
+        _wsgi_server.db.close()
+
+
+#atexit.register(cleanup)
+
+
 def wsgi_application(dbpath, basepath):
     """Return a WSGI application object with a database at the
     specified path.
@@ -510,6 +530,7 @@ def wsgi_application(dbpath, basepath):
     is the same as the first argument to Apache's WSGIScriptAlias
     directive.
     """
+
     def application(environ, start_response):
         global _wsgi_server
         if _wsgi_server is None:
@@ -543,4 +564,5 @@ def wsgi_application(dbpath, basepath):
 
         # Call the normal application
         return _wsgi_server.wsgi_application(environ, start_response)
+
     return application
