@@ -26,7 +26,9 @@ class Timestamper():
                 return b""
             if line[0:1] == b'#':
                 continue
-            break
+            # For some reason, coverage on python 3.8 reports that
+            # we never hit this break, even though we definitely do.
+            break  # pragma: no cover
         try:
             return next(self.ts_iter) + line
         except StopIteration:
